local events, shovelButton, _, T = CreateFrame("FRAME"), nil, ...
local NINE = select(4,GetBuildInfo()) > 9e4

local function UnitNPCID(unit)
	local guid = UnitGUID(unit)
	return guid and tonumber((guid:match("^Creature%-.-(%d+)%-%x*$")))
end

local FARM_MIN_LEVEL = NINE and 15 or 80 -- Don't activate for low-level players
events:RegisterEvent("PLAYER_LOGIN")
events:RegisterEvent("PLAYER_LEVEL_UP")
events:SetScript("OnEvent", function(self, event, level)
	if (event == "PLAYER_LOGIN" and UnitLevel("player") >= FARM_MIN_LEVEL) or
	   (event == "PLAYER_LEVEL_UP" and level >= FARM_MIN_LEVEL) then
		self:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
		self:UnregisterEvent("PLAYER_LEVEL_UP")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		if not InCombatLockdown() then
			self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		end
		shovelButton = CreateFrame("BUTTON", "SpadeActionButton", nil, "SecureActionButtonTemplate")
		shovelButton:SetAttribute("type", "macro")
		shovelButton:SetAttribute("macrotext", "/target mouseover\n/use item:89880")
		if T.InitRing then T.InitRing() end
		if T.InitTracker then T.InitTracker() end
	end
	if event == "PLAYER_LOGIN" and T.TrackerOnLogin then T.TrackerOnLogin() end
end)

local afflictionAuras = {[115483] = 80513, [115824] = 79104, [132726] = 89880, [116114] = 89815}
local plants = {} for id in ("0E4C60F6B10F6B90F6CD0F6F80F70E0F71D1017C101AD101C2101D3101DD101F8102211023D1024C1027B1027C1028D1079A"):gmatch(".....") do plants[id] = true end

-- Add a line to the tooltip, if showing the mouseover unit
function T.addActionHint(tip, text, prefix)
	local _, unit = tip:GetUnit()
	if unit and UnitIsUnit(unit, "mouseover") then
		tip:AddLine((prefix or "") .. (prefix ~= false and "|TInterface\\TUTORIALFRAME\\UI-TUTORIAL-FRAME:::0:0:512:512:0:80:328:412|t: " or "") .. (text or ""))
		tip:Show()
	end
end

-- UPDATE_MOUSEOVER_UNIT does not fire when mouseover disappears, so poll UnitExists and fire it manually
local function watchMouseover(self)
	if not UnitExists("mouseover") then
		self:SetScript("OnUpdate", nil)
		self:GetScript("OnEvent")(self, "UPDATE_MOUSEOVER_UNIT")
	end
end

function events:PLAYER_REGEN_DISABLED()
	self:SetScript("OnUpdate", nil)
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	ClearOverrideBindings(self)
	-- No T.UnbindRing() here: ring override bindings automatically cleared in combat
end
function events:PLAYER_REGEN_ENABLED()
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:UPDATE_MOUSEOVER_UNIT()
end
function events:UPDATE_MOUSEOVER_UNIT()
	local npc = UnitNPCID("mouseover")
	
	-- Check for Tilled Soil, and bind/unbind OPie ring
	local isTilledSoil = npc == 0xE4C3 or npc == 0xEA56
	if isTilledSoil then
		if T.BindRing(GameTooltip) then
			self:SetScript("OnUpdate", watchMouseover)
		end
	else
		T.UnbindRing()
	end
	
	-- Check for Infested/Parched/Growing crops and Untilled Soil, bind the relevant tool
	ClearOverrideBindings(self)
	local toolItem = (not isTilledSoil) and npc and afflictionAuras[select(10, UnitAura("mouseover", 1))]
	toolItem = toolItem or (npc == 58562 and 89815)
	if toolItem and GetItemCount(toolItem) > 0 then
		self:SetScript("OnUpdate", watchMouseover)
		if toolItem == 89880 then
			SetOverrideBindingClick(self, true, "ALT-BUTTON2", shovelButton:GetName())
		else
			SetOverrideBindingItem(self, true, toolItem == 89815 and "ALT-BUTTON2" or "BUTTON2", "item:" .. toolItem)
		end
		T.addActionHint(GameTooltip, T.L["use:" .. toolItem], (toolItem == 89815 or toolItem == 89880) and "|cffcececeAlt + |r" or "")
	elseif toolItem and toolItem ~= 89815 then
		T.addActionHint(GameTooltip, RED_FONT_COLOR_CODE .. (T.L["tool:" .. toolItem]):gsub("%b[]", HIGHLIGHT_FONT_COLOR_CODE .. "%1" .. RED_FONT_COLOR_CODE), false)
	end
	if toolItem == 89880 and T.HintFarmReset then
		T.HintFarmReset(select(6, UnitAura("mouseover", 1))-GetTime())
	end
end