local _, T = ...

local ORL, ORI, AB, collection, collectionID, bagCollection, bagCollectionID
local seedIDs = {79102, 80590, 80591, 80592, 80593, 80594, 80595, 85215, 85216, 85217, 85267, 85268, 85269, 89197, 89202, 89233, 89326, 89328, 89329, 85219, 80302, 91806}
local bagIDs = {[80809]=79102, [95434]=79102, [84782]=80590, [95436]=80590, [84783]=80591, [95441]=80591, [85153]=80592, [95438]=80592, [85158]=80593, [95440]=80593, [85162]=80594, [95439]=80594, [85163]=80595, [95443]=80595, [95447]=85215, [95448]=85215, [95449]=85216, [95450]=85216, [95451]=85217, [95452]=85217, [95454]=89197, [95456]=89197, [95457]=89202, [95458]=89202, [95445]=89233, [95446]=89233, [89847]=89326, [95444]=89326, [89848]=89328, [95437]=89328, [89849]=89329, [95442]=89329}
local seedQuests = {[31942]=89329, [31672]=80592, [31673]=80593, [31941]=89328, [31670]=80590, [31669]=79102, [31674]=80594, [31675]=80595, [31943]=89326, [31671]=80591, [31946]=80590, [31947]=79102, [30527]=89329, [31949]=89326, [30254]=80302, [31945]=80591, [32658]=80592, [32653]=89329, [32649]=79102, [32645]=89326, [32647]=80595, [32642]=80593}

local NINE = select(4, GetBuildInfo()) > 9e4

local GetQuestLogTitle_NINE = NINE and function(i)
	local q = C_QuestLog.GetInfo(i)
	if q then
		local qid = q.questID
		return nil, nil, nil, q.isHeader, q.isCollapsed, C_QuestLog.IsComplete(qid), nil, qid
	end
end


-- Check if there are uncompleted plant-a-seed quests; save required seed IDs
local questSeeds = {}
local	function scanQuestSeeds(i)
	if i == nil then wipe(questSeeds) end
	local GetQuestLogTitle = GetQuestLogTitle_NINE or GetQuestLogTitle
	for i=i or 1, NINE and C_QuestLog.GetNumQuestLogEntries() or GetNumQuestLogEntries() do
		local _, _, _, _, isHeader, isCollapsed, _, _, qid = GetQuestLogTitle(i)
		if isHeader and isCollapsed then
			ExpandQuestHeader(i)
			scanQuestSeeds(i+1)
			CollapseQuestHeader(i)
			return
		elseif seedQuests[qid] and not select(3, GetQuestLogLeaderBoard(qid == 30254 and 2 or 1, i)) then
			questSeeds[seedQuests[qid]] = true
		end
	end
end

-- Rescan bags whenever ActionBook tells us one of the collections is about to be displayed
local bagCharges = {}
local function syncBagCollection()
	scanQuestSeeds()
	wipe(bagCharges)
	
	local n, collection = 1, bagCollection
	for iid, sid in pairs(bagIDs) do
		local charges, token, action = GetItemCount(iid, false, true), "SpadeBag" .. sid, nil
		if charges > 0 then
			charges, action = (charges - 1) % 10 + 1, AB:GetActionSlot("item", iid)
		end
		if action then
			if bagCharges[token] == nil then
				collection[n], n, collection[token], bagCharges[token] = token, n + 1, action, charges
				ORI:SetQuestHint(token, questSeeds[sid])
			elseif bagCharges[token] > charges then
				collection[token], bagCharges[token] = action, charges
			end
		elseif not bagCharges[token] then
			collection[token] = nil
		end
	end
	for i=n, #collection do
		collection[i] = nil
	end
	table.sort(collection)
	AB:UpdateActionSlot(bagCollectionID, collection)

	-- If user hasn't told OPie otherwise, display this ring centered on the mouse, not screen
	if select(3, ORL:GetOption("RingAtMouse", "Spade.BagRing")) == nil then
		ORL:SetOption("RingAtMouse", true, "Spade.BagRing")
	end
end
local function syncCollection(_, _, openId)
	if openId ~= collectionID then
		return openId == bagCollectionID and syncBagCollection()
	end
	scanQuestSeeds()
	
	local n, changed = 1, nil
	for i=1,#seedIDs do local iid = seedIDs[i]
		local action, token = AB:GetActionSlot("item", iid), "SpadeSeed" .. iid
		if action then
			collection[n], collection[token], n, changed = token, action, n + 1, changed or (collection[n] ~= token)
			ORI:SetQuestHint(token, questSeeds[iid])
		else
			collection[token], changed = nil, changed or (collection[token] ~= nil)
		end
	end
	
	if changed then
		for i=n, #collection do
			collection[i] = nil
		end
		AB:UpdateActionSlot(collectionID, collection)
	end
	
	-- If user hasn't told OPie otherwise, display this ring centered on the mouse, not screen
	if select(3, ORL:GetOption("RingAtMouse", "Spade.Ring")) == nil then
		ORL:SetOption("RingAtMouse", true, "Spade.Ring")
	end
end

function T.InitRing()
	ORL = OneRingLib
	ORI = OPie and OPie.UI or ORL and ORL.ext and ORL.ext.OPieUI
	AB = ORL and ORL.ext and ORL.ext.ActionBook and ORL.ext.ActionBook:compatible(2, 15)
	if not (ORL and ORI and AB) then
		ORL = nil
		return false
	end
	collection, bagCollection = {__openAction=AB:GetActionSlot("macrotext", "/target mouseover")}, {}
	collectionID, bagCollectionID = AB:CreateActionSlot(nil, nil, "collection", collection), AB:CreateActionSlot(nil, nil, "collection", bagCollection)
	AB:AddObserver("internal.collection.preopen", syncCollection)
	ORL:SetRing("Spade.Ring", collectionID, {name=T.L"Spade Seeds", internal=true, noOpportunisticCA=true, noPersistentCA=true})
	ORL:SetRing("Spade.BagRing", bagCollectionID, {name=T.L"Spade Seed Bags", noOpportunisticCA=true, noPersistentCA=true})
	ORI:SetDisplayOptions("SpadeSeed89329", nil, nil, 0.29, 0.81, 0.75)
	ORI:SetDisplayOptions("SpadeSeed85215", nil, nil, 0, 0.81, 0.62)
end
local knownModes, modeBind = {["only-seeds"]=3, ["only-bags"]=1, ["invert"]=7, ["none"]=0}, {"BUTTON2", "ALT-BUTTON2"}
function T.BindRing(Tooltip)
	if ORL and ORL.OverrideRingBinding then
		local mode = type(SpadeOptions) == "table" and knownModes[SpadeOptions.RingMode] or 5
		local mSeed, mBag = math.floor(mode/3), mode % 3
		ORL:OverrideRingBinding("Spade.Ring", modeBind[mSeed])
		ORL:OverrideRingBinding("Spade.BagRing", modeBind[mBag])
		if mSeed == 1 or mBag == 1 then
			T.addActionHint(Tooltip, T.L(mBag ~= 1 and "Plant a seed" or "Use a seed bag"), "")
		end
		if mSeed == 2 or mBag == 2 then
			T.addActionHint(Tooltip, T.L(mBag ~= 2 and "Plant a seed" or "Use a seed bag"), "|cffcececeAlt + |r")
		end
		return mode > 0
	elseif not (type(SpadeOptions) == "table" and SpadeOptions.RingMode == "none") then
		T.addActionHint(Tooltip, RED_FONT_COLOR_CODE .. (T.L"Requires %s to select seeds"):format(HIGHLIGHT_FONT_COLOR_CODE .. "OPie" .. RED_FONT_COLOR_CODE), false)
	end
end
function T.UnbindRing()
	if ORL and ORL.OverrideRingBinding then
		ORL:OverrideRingBinding("Spade.Ring", nil)
		ORL:OverrideRingBinding("Spade.BagRing", nil)
	end
end