local me, _, T, _gossip, _data = UnitName("player"), ...
SpadeUI:SetScript("OnEvent", function(self)
	self:UnregisterEvent("GET_ITEM_INFO_RECEIVED")
	T.showSpadeUI(true, _gossip, _data)
end)

local HARVESTABLE_COLOR_CODE = GREEN_FONT_COLOR_CODE
local FUTURE_COLOR_CODE = "|cffE6A22E"

local safepairs do
	local function noop()
	end
	function safepairs(t)
		return type(t) == "table" and next or noop, t
	end
end

local function updateIcon(seed, iid, tex, count, isFuture, extraLine, title)
	seed.icon:SetTexture(tex)
	seed.count:SetText(count)
	seed.overlay:SetShown(isFuture)
	seed.iid, seed.extraLine, seed.title = iid, extraLine, title
	if not tex then SpadeUI:RegisterEvent("GET_ITEM_INFO_RECEIVED") end
end
local function updateFreeIcon(ico, nfree, cap)
	return updateIcon(ico, nil, [[Interface\Icons\INV_Misc_Dust_02]], nfree, false, cap, T.L"Unused Soil Plots")
end
local function updateCharacter(char, data, name)
	if type(data) ~= "table" then return end
	local c, cf, nextIcon = 0, 0, 1

	for k,v in pairs(data) do
		if type(k) == "number" and type(v) == "number" then
			updateIcon(char.seeds[nextIcon], k, GetItemIcon(k), HARVESTABLE_COLOR_CODE .. v)
			c, nextIcon = c + v, nextIcon + 1
		end
	end
	for k,v in safepairs(data.future) do
		updateIcon(char.seeds[nextIcon], k, GetItemIcon(k), FUTURE_COLOR_CODE .. v, true)
		cf, nextIcon = cf + v, nextIcon + 1
	end
	if data.sz and (cf + c) < data.sz then
		updateFreeIcon(char.seeds[nextIcon], data.sz - cf - c)
		nextIcon = nextIcon + 1
	end
	char:SetHeight(39 * math.ceil((nextIcon-1) / 6) + 20)
	for i=1,#char.seeds do char.seeds[i]:SetShown(i < nextIcon) end
	char.title:SetText(name)
	local s = c > 0 and (HARVESTABLE_COLOR_CODE .. c .. "|r") or ""
	if cf > 0 then s = s .. (s == "" and "" or "+") .. FUTURE_COLOR_CODE .. cf .. "|r" end
	char.size:SetText(s)
	return nextIcon > 1
end
local function sum(t)
	local sum, cap = 0, ""
	for k,v in pairs(t) do
		sum, cap = sum + v, ("%s%s%s %s(%d)|r"):format(cap, cap == "" and "" or ", ", k, HIGHLIGHT_FONT_COLOR_CODE, v)
	end
	return sum, cap
end
local function updateCompound(char, data)
	if type(data) ~= "table" then return end
	local cur, future, free, ncur, nfuture, nfree = {}, {}, {}, 0, 0, 0
	for k,v in safepairs(data) do
		local isEntry, nused = type(v) == "table", 0
		for id, c in safepairs(v) do
			if type(id) == "number" and type(c) == "number" then
				cur[id] = cur[id] or {}
				cur[id][k], nused, ncur = c, nused + c, ncur + c
			end
		end
		for id, c in safepairs(isEntry and v.future) do
			if type(id) == "number" and type(c) == "number" then
				future[id] = future[id] or {}
				future[id][k], nused, nfuture = c, nused + c, nfuture + c
			end
		end
		if isEntry and type(v.sz) == "number" and v.sz > nused then
			free[k], nfree = v.sz - nused, nfree + (v.sz - nused)
		end
	end
	if (ncur + nfuture + nfree) == 0 then return false end
	
	local nextIcon = 1
	for iid, ch in pairs(cur) do
		local sum, cap = sum(ch)
		updateIcon(char.seeds[nextIcon], iid, GetItemIcon(iid), HARVESTABLE_COLOR_CODE .. sum, false, cap)
		nextIcon = nextIcon + 1
	end
	for iid, ch in pairs(future) do
		local sum, cap = sum(ch)
		updateIcon(char.seeds[nextIcon], iid, GetItemIcon(iid), FUTURE_COLOR_CODE .. sum, true, cap)
		nextIcon = nextIcon + 1
	end
	if nfree > 0 then
		updateFreeIcon(char.seeds[nextIcon], sum(free))
		nextIcon = nextIcon + 1
	end
	char:SetHeight(39 * math.ceil((nextIcon-1) / 6) + 22)
	for i=1,#char.seeds do char.seeds[i]:SetShown(i < nextIcon) end
	char.title:SetText(T.L"On this realm")
	local s = ncur > 0 and (HARVESTABLE_COLOR_CODE .. ncur .. "|r") or ""
	if nfuture > 0 then s = s .. (s == "" and "" or "+") .. FUTURE_COLOR_CODE .. nfuture .. "|r" end
	char.size:SetText(s)

	return true
end

function T.showSpadeUI(updateOnly, gossip, data)
	if updateOnly and not SpadeUI:IsShown() then return end
	_gossip, _data = gossip, data
	SpadeUI.TitleText:SetText(T.L"Sunsong Ranch Plants")
	local nextChar = data[me] and updateCharacter(SpadeUI.chars[1], data[me], me) and 2 or 1
	if updateCompound(SpadeUI.chars[nextChar], data) then nextChar = nextChar + 1 end

	for i=1,#SpadeUI.chars do SpadeUI.chars[i]:SetShown(i < nextChar) end
	local range = (nextChar > 1 and (SpadeUI.chars[1]:GetTop() - SpadeUI.chars[nextChar-1]:GetBottom()) or 0) + 5
	SpadeUI.gossip:SetText(gossip or "")
	SpadeUI.scroll:SetPoint("TOPLEFT", SpadeUI.gossip, "BOTTOMLEFT", 0, gossip and -5 or 0)
	SpadeUI.scroll:SetHeight(math.min(365-32-10, range))
	SpadeUI.scroll.child:SetHeight(range)
	SpadeUI:SetHeight(math.min(365, 32 + range + 10) + (gossip and (SpadeUI.gossip:GetHeight() + 5) or 0))

	SpadeUI:Show()
end

function T.AnnounceHarvest(mc, c)
	if c > 0 and not (type(SpadeOptions) == "table" and SpadeOptions.Mute) then
		local cnt = mc > 0 and mc or c
		local m = T.L(mc > 0 and "%d |4plant:plants; ready for harvest: [View Farms]." or "%d |4plant:plants; ready for harvest on alts: [View Farms]."):gsub("|4(.-):(.-);", cnt == 1 and "%1" or "%2"):format(cnt)
		print(NORMAL_FONT_COLOR_CODE .. m:gsub("(%b[])", HIGHLIGHT_FONT_COLOR_CODE .. "|Hgarrmission:nam:spade|h%1|h" .. NORMAL_FONT_COLOR_CODE))
	end
end
hooksecurefunc("SetItemRef", function(link)
	if link == "garrmission:nam:spade" then
		return T.updateAndShowSpadeUI()
	end
end)

local function setRingMode(_, arg1)
	SpadeOptions = type(SpadeOptions) == "table" and SpadeOptions or {}
	SpadeOptions.RingMode = arg1
end
local function toggleMute()
	SpadeOptions = type(SpadeOptions) == "table" and SpadeOptions or {}
	SpadeOptions.Mute = not SpadeOptions.Mute or nil
end
local function clearFarm()
	local t = T.GetPlayerFarm()
	wipe(t)
	local sz = T.GetFarmSize()
	if sz and sz > 0 then t.sz = sz end
	T.updateAndShowSpadeUI(true)
end
local pocketMenu
SpadeUI.config:SetScript("OnClick", function()
	if not pocketMenu then
		pocketMenu = CreateFrame("FRAME", "SpadeUIConfigDropdown", UIParent, "UIDropDownMenuTemplate")
		UIDropDownMenu_Initialize(pocketMenu, function(_, level)
			local info = UIDropDownMenu_CreateInfo()
			if level == 1 then
				info.text, info.isNotRadio, info.checked, info.func = T.L"Announce harvest on log in", true, not (SpadeOptions and SpadeOptions.Mute), toggleMute
				UIDropDownMenu_AddButton(info)
				info.text, info.notCheckable, info.func = T.L"Clear current farm", true, clearFarm
				if T.PlayerHasPlants() or IsAltKeyDown() then UIDropDownMenu_AddButton(info) end
				info.text, info.notCheckable, info.hasArrow, info.func = T.L"Bind OPie rings", true, true
				UIDropDownMenu_AddButton(info)
			else
				local cur = SpadeOptions and SpadeOptions.RingMode or "default"
				info.text, info.checked, info.func, info.arg1 = T.L"Default (prefer seeds)", cur == "default", setRingMode, nil
				UIDropDownMenu_AddButton(info, level)
				info.text, info.checked, info.arg1 = T.L"Inverted (prefer bags)", cur == "invert", "invert"
				UIDropDownMenu_AddButton(info, level)
				info.text, info.checked, info.arg1 = T.L"Only seeds", cur == "only-seeds", "only-seeds"
				UIDropDownMenu_AddButton(info, level)
				info.text, info.checked, info.arg1 = T.L"Only seed bags", cur == "only-bags", "only-bags"
				UIDropDownMenu_AddButton(info, level)
				info.text, info.checked, info.arg1 = T.L"None", cur == "none", "none"
				UIDropDownMenu_AddButton(info, level)
			end
		end, "MENU")
	end
	ToggleDropDownMenu(1, nil, pocketMenu, "cursor", 3, -3)
end)
SpadeUI.config:SetScript("OnHide", function()
	if UIDROPDOWNMENU_OPEN_MENU == pocketMenu then
		CloseDropDownMenus()
	end
end)

SLASH_SPADE1, SLASH_SPADE2 = "/spade", T.L"/spade"
function SlashCmdList.SPADE()
	T.updateAndShowSpadeUI()
end